package mastermind.client;

import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.File;
import javax.swing.*;
import mastermind.logic.SettingsHandler;
import mastermind.model.Settings;
import mastermind.solver.SolverLogik;

/**
 * Mastermind GUI.
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-11.v01   ReMa    init version
 * 2012-05-13.v01   ReMa    div Neuerungen (MouseListener, Results, etc.)
 * 2012-05-19.v01   FeRo    Load / Save hinzugefügt
 * -----------------------------------------------------------------------------
 * @author Marco Reichmuth <marco.reichmuth@stud.hslu.ch>
 * @version 2012-05-19.v01
 */
public class ClientGUIActionListener implements ActionListener
{

    private ClientGUI gui;

    /**
     * Default Konstruktor
     * @param gui Mastermind GUI
     */
    public ClientGUIActionListener(ClientGUI gui)
    {
        this.gui = gui;
    }

    /**
     * actionPerformed Switch für die einzelnen Menu Punkte
     * @param e ActionEvent
     */
    public void actionPerformed(ActionEvent e)
    {
        String tmp = e.getActionCommand();
        if (tmp.equals("New")) {
            gui.startNewGame(true);
        } else if (tmp.equals("New Solver")) {
            if (SolverLogik.solvableSettings()) {
                gui.startNewSolver();
            } else {
                JOptionPane.showMessageDialog(gui, "I could solve this, but not on this hardware... ;-)\nPlease choose other settings.", "Solver", JOptionPane.ERROR_MESSAGE);
            }
        } else if (tmp.equals("Open")) {
            showLoadDialog();
        } else if (tmp.equals("Save")) {
            showSaveDialog();
        } else if (tmp.equals("Settings")) {
            showSettings();
        } else if (tmp.equals("Exit")) {
            System.exit(0);
        } else if (tmp.equals("Show one hidden Peg")) {
            gui.showHiddenPeg();
        } else if (tmp.equals("Show all hidden Pegs")) {
            gui.showHiddenPattern();
        } else if (tmp.equals("Help")) {
            showHelpDialog();
        } else if (tmp.equals("About")) {
            showAboutDialog();
        } else if (tmp.equals("Random")) {
            gui.generateRandom();
            gui.validateNewHiddenGuess();
        } else {
            gui.chg_button();
        }

    }

    /**
     * Settings-Dialog anzeigen
     */
    private void showSettings()
    {
        Settings oldSettings = SettingsHandler.getInstance().getSettings();
        int anzahlFarben = oldSettings.getNumberOfColors();
        boolean doppelteBFarben = oldSettings.getAllowDuplicateColor();
        int anzahlSteckplaetze = oldSettings.getNumberOfPegs();
        int anzahlVersuche = oldSettings.getNumberOfRows();
        int design = oldSettings.getDesign();

        //System.out.println("Show Settings Dialog...");
        JDialog myDialog = new JDialog(gui, "Settings", true);
        myDialog.add(new ClientGUISettings(myDialog));
        myDialog.setSize(480, 210);
        myDialog.setResizable(false);
        myDialog.setLocationRelativeTo(gui);
        myDialog.setVisible(true);
        // wait till close....
        //System.out.println("Show Settings Dialog - finish...");
        myDialog.dispose();

        // validate Settings
        Settings newSettings = SettingsHandler.getInstance().getSettings();
        if ( ! SettingsHandler.getInstance().validSettings()) {
            // restore old Settings
            System.out.println("no valid settings => restore old config...");
            try {
                newSettings.setNumberOfColors(anzahlFarben);
                newSettings.setAllowDuplicateColor(doppelteBFarben);
                newSettings.setNumberOfPegs(anzahlSteckplaetze);
                newSettings.setNumberOfRows(anzahlVersuche);
                newSettings.setDesign(design);
            } catch (Exception e) {
                // ignore
            }
        }

        if ((anzahlFarben == newSettings.getNumberOfColors()) && (doppelteBFarben == newSettings.getAllowDuplicateColor()) && (anzahlSteckplaetze == newSettings.getNumberOfPegs()) && (anzahlVersuche == newSettings.getNumberOfRows())) {
            // check design change
            if (design != newSettings.getDesign()) {
                // new design - leave gameEngine as is
                System.out.println("new Design...");
                gui.startNewGame(false);
            } else {
                System.out.println("nothing has been changed => do nothing");
            }
        } else {
            // ini new game - reset GameEngine
            gui.startNewGame(true);
        }
    }

    /**
     * Dialog anzeigen in welche Datei das Spiel gespeichert werden soll.
     */
    private void showSaveDialog()
    {
        // gui.saveGame(new File("test.dat"));
        final JFileChooser fc = new JFileChooser();
        fc.addChoosableFileFilter(new MMFileFilter());
        fc.setSelectedFile(new File("mygame.dat"));
        int returnVal = fc.showSaveDialog(gui);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            gui.saveGame(fc.getSelectedFile());
            JOptionPane.showMessageDialog(gui, "Game has been saved.", "Save", JOptionPane.INFORMATION_MESSAGE);
        } else {
            // cancelled by user
        }
    }

    /**
     * Dialog anzeigen in welche Datei das Spiel gespeichert werden soll.
     */
    private void showLoadDialog()
    {
        //gui.loadGame(new File("test.dat"));
        final JFileChooser fc = new JFileChooser();
        fc.addChoosableFileFilter(new MMFileFilter());
        fc.setSelectedFile(new File("mygame.dat"));
        int returnVal = fc.showOpenDialog(gui);
        if (returnVal == JFileChooser.APPROVE_OPTION) {
            gui.loadGame(fc.getSelectedFile());
        } else {
            // cancelled by user
        }
    }

    /**
     * Philipp Rey
     */
    private void showHelpDialog()
    {
        new ClientGUIHelp().setVisible(true);
    }

    /**
     * Philipp Rey
     */
    private void showAboutDialog()
    {
        JOptionPane.showMessageDialog(gui, "Mastermind v1.5 \n\n" +
            "\u00a9 2012  Iljasa Redzepi, Marco Reichmuth, \nFelix Rohrer," +
            " Philipp Rey", "About", JOptionPane.INFORMATION_MESSAGE);
    }
}
