package mastermind.model;

import java.io.Serializable;

/**
 * Mastermind Result: Hilfsobjekt für Bewertung von einem Guess.
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-02.v01   FeRo    init version
 * 2012-05-03.v01   FeRo    incAnzPegColorPosRight / incAnzPegColorRight
 * 2012-05-19.v01   FeRo    Serializable
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-19.v01
 */
public class Result implements Serializable
{
    // Klassenvariablen für die Anzahl Stifte (Schwarz / Weiss)

    private int anzPegColorPosRight = 0; // Anzahl Stifte: richtige Farbe und richtige Position => Schwarz
    private int anzPegColorRight = 0; // Anzahl Stifte: richtige Farbe => Weiss

    /**
     * Default Konstruktor - empty
     */
    public Result()
    {
        // nothing
        // per Default sind beide 0
    }

    /**
     * Konstruktor für Result
     * @param anzPegColorPosRight Anzahl Stifte mit richtiger Farbe und an richtiger Position
     * @param anzPegColorRight Anzahl Stifte mit richter Farbe
     */
    public Result(int anzPegColorPosRight, int anzPegColorRight)
    {
        setAnzPegColorPosRight(anzPegColorPosRight);
        setAnzPegColorRight(anzPegColorRight);
    }

    /**
     * Gibt die Anzahl Stifte mit richtiger Farbe und richtiger Position zurück => Schwarz
     * @return Anz. Stifte mit Farbe & Position richtig
     */
    public int getAnzPegColorPosRight()
    {
        return anzPegColorPosRight;
    }

    /**
     * Setzt die Anzahl Stifte mit der richtigen Farbe und richtiger Position
     * @param anzPegColorPosRight Anzahl Stifte
     */
    public void setAnzPegColorPosRight(int anzPegColorPosRight)
    {
        if (anzPegColorPosRight < 0) {
            anzPegColorPosRight = 0;
        }
        this.anzPegColorPosRight = anzPegColorPosRight;
    }

    /**
     * Gibt die Anzahl Stifte mit der richtigen Farbe zurück => weiss
     * @return Anz. Stifte mit Farbe richtig
     */
    public int getAnzPegColorRight()
    {
        return anzPegColorRight;
    }

    /**
     * Setzt die Anzahl Stifte mit der richtigen Farbe
     * @param anzPegColorRight Anz. Stifte mit richtiger Farbe
     */
    public void setAnzPegColorRight(int anzPegColorRight)
    {
        if (anzPegColorRight < 0) {
            anzPegColorRight = 0;
        }
        this.anzPegColorRight = anzPegColorRight;
    }

    /**
     * Increment Anzahl Stifte an richtiger Position mit richtiger Farbe
     */
    public void incAnzPegColorPosRight()
    {
        this.anzPegColorPosRight ++;
    }

    /**
     * Increment Anzahl Stifte mit richtiger Farbe
     */
    public void incAnzPegColorRight()
    {
        this.anzPegColorRight ++;
    }

    /**
     * Gibt die Anzahl Schwarze und Weisse Stiffte als Array zurück.
     * @return int[] (Anz.Color&Farbe richtig, Anz.Farbe richtig)
     */
    public int[] getResult()
    {
        return (new int[]{anzPegColorPosRight, anzPegColorRight});
    }

    /**
     * Gibt als String das Resultat zurück.
     * @return String mit Anzahl Schwarzen und Weisse Stiften
     */
    public String getResultAsString()
    {
        return ("Schwarz: " + anzPegColorPosRight + ", Weiss: " + anzPegColorRight);
    }

    /**
     * Gibt die Anzahl Schwarze und Weisse Stifte als String zurück
     * @return Schwarze und Weisse Stifte
     */
    @Override
    public String toString()
    {
        return this.getResultAsString();
    }

    /**
     * Vergleich zwei Guess miteinander. Gibt True zurück wenn sie gleich sind.
     * @param otherResult zu vergleichender Guess
     * @return gleicheit der zwei Guess
     */
    @Override
    public boolean equals(Object otherResult)
    {
        if (this == otherResult) {
            return true;
        }
        if (otherResult == null) {
            return false;
        }
        if (otherResult.getClass() != this.getClass()) {
            return false;
        }
        if ( ! this.toString().equals(((Result) otherResult).toString())) {
            return false;
        }
        return true;
    }

    /**
     * Berechent den HashCode
     * @return HashCode des aktuellen Guess
     */
    @Override
    public int hashCode()
    {
        int hash = 3;
        hash = 53 * hash + this.anzPegColorPosRight;
        hash = 53 * hash + this.anzPegColorRight;
        return hash;
    }
}
