package mastermind.solver;

import java.util.ArrayList;
import java.util.List;
import mastermind.logic.SettingsHandler;
import mastermind.model.Guess;
import mastermind.model.Result;

/**
 * Mastermind Solver: Thread Class für alle möglichen Guesses berechnen
 */
/* -----------------------------------------------------------------------------
 * Version History:
 * 2012-05-25.v01   FeRo    init version
 * -----------------------------------------------------------------------------
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 * @version 2012-05-25.v01
 */
public class SolverThreadGenAllGuess implements Runnable
{
    private int threadNum;
    private SolverLogik logik;
    private int startPos;
    private int endPos;
    private int numberOfColors;
    private int numberOfPegs;
    private Guess firstGuess;
    private Result firstResult;

    /**
     * Default Konstruktor
     * @param logik Solver Logik
     * @param startPos StartPosition
     * @param endPos EndPosition
     * @param firstGuess 1. Guess
     * @param firstResult 1. Resultat
     * @param id ID vom Thread
     */
    public SolverThreadGenAllGuess(SolverLogik logik, int startPos, int endPos, Guess firstGuess, Result firstResult, int id)
    {
        this.threadNum = id;
        this.logik = logik;
        this.startPos = startPos;
        this.endPos = endPos;
        this.firstGuess = firstGuess;
        this.firstResult = firstResult;
        this.numberOfColors = SettingsHandler.getInstance().getSettings().getNumberOfColors();
        this.numberOfPegs = SettingsHandler.getInstance().getSettings().getNumberOfPegs();
    }

    /**
     * Thread run() - eigentlicher Code
     */
    public void run()
    {
        List<Guess> moves = new ArrayList<Guess>(endPos-startPos);
        //System.out.println(" >> " + threadNum + " GenAllGuess... (" + startPos + " - " + endPos + ")");
        int intTmp = 0;
        Guess newGuess;
        for (int moveIndex = startPos; moveIndex < endPos; moveIndex ++) {
            int[] arrColor = new int[numberOfPegs];
            intTmp = moveIndex;
            for (int i = numberOfPegs - 1; i >= 0; i --) {
                arrColor[i] = (intTmp % numberOfColors) + 1;
                intTmp = (int) Math.floor(intTmp / (double) numberOfColors);
            }
            newGuess = new Guess(arrColor);
            // System.out.println(moveIndex + ": " + newGuess);
            // Is this Guess a possible Guess for the current Pattern?

            Result result = firstGuess.compareGuess(newGuess);
            if (result.equals(firstResult)) {
                // same result, this guess could be the solution
                moves.add(newGuess);
            }
        }
        //System.out.println(threadNum + ": Add: " + moves.size());
        logik.setListGuess(moves);
    }
}
